package resource.util;

import java.net.*;
import java.util.*;

/**
 * The global manager for all resources. The ResourceManager is responsible for managing all binary resources
 * of the framework. This can be images, videos, text ...<br>
 * <br>
 * Resources are managed as follows:<br>
 * <br>
 * The root directory is a directory ending with the name "resource". Here it is just the directory
 * "resource". Starting from this directory all resources are given back relative to this path.<br>
 * <br>
 * There can be multiple resouces to manage: gif-images, jpeg-images, text and others.<br>
 * For all of these resources there should exist one resource module.<br>
 * There is already one module for {@link #RESOURCE_GIF gif-resources}.<br>
 * </br>
 * Resources are requested as follows:<br>
 * <CODE>URL url = ResourceManager.getInstance().getResource (String type, String path);</CODE></br>
 * The "type" is the name of the resource module. The path string is the path relative to the directory
 * of the requested type and is separated with a dot.
 * </br>
 * Example:</br>
 * </br>
 * An icon of the size 16x16 should be requested. It is located in the directory "resource/gif/icon".<br>
 * The icon's name is "document.gif".
 * So the call is:<br>
 * <CODE>URL url = ResourceManager.getInstance().getResource (RESOURCE_GIF, "icon.document");</CODE>
 *
 * @author Thomas Medack
 * @version 1.0
 */
public class ResourceManager {

    /**
     * Interface for a resource module.
     */
    public static interface ResourceModul {
        /**
         * Returns the path relative to the current resource path.
         *
         * @return the path.
         */
        public String getPath();

        /**
         * Returns the file extension of the resource, e.g. "gif".
         *
         * @return the file extension
         */
        public String getExtension();

        /**
         * Returns the type name of the module. This is the name by which the resources are requested.
         *
         * @return the type name
         */
        public String getTypeName();
    }

    /**
     * Map of all resource modules.
     */
    private HashMap moduls = new HashMap();

    /**
     * The root path of all resources.
     */
    protected String resourceRootPath = "";

    /**
     * The singleton instance of the resource manager.
     */
    private static ResourceManager instance = null;

    /**
     * Creates a new instance of ResourceManager.
     */
    protected ResourceManager() {
        // add resource module
        addResourceModul(new ResourceModulGIF());

        // root path is package "resource"
        resourceRootPath = "resource";
    }

    /**
     * Returns the singleton instance of the ResourceManager.
     *
     * @return the singleton instance.
     */
    public static ResourceManager getInstance() {
        if (instance == null) {
            instance = new ResourceManager();

        }
        return instance;
    }

    /**
     * Adds a resource module to the ResourceManager.
     *
     * @param rm the resource module to be added.
     */
    public void addResourceModul(ResourceModul rm) {
        moduls.put(rm.getTypeName(), rm);
    }

    /**
     * Removes a resource module from the ResourceManager.
     *
     * @param rm the resource module to be removed.
     */
    public void removeResourceModul(ResourceModul rm) {
        moduls.remove(rm.getTypeName());
    }

    /**
     * Returns a resource as URL. Therefor the logical name of the resource module and the relative path
     * starting from it have to be specified.<br>
     * </br>
     * Example:</br>
     * </br>
     * logical name of the gif-module: "RESOURCE_GIF". The absolute path of this module is
     * "org/eit/ui/resource/gif". The requested resource is "document.gif".<br>
     * Therefore the call is: <CODE>getResource ("RESOURCE_GIF", "icon.16x16.document)</CODE>
     * (without file extension).
     *
     * @param type the logical type name
     * @param path der relative path + file name without extension
     * @return the resource's URL.
     */
    public URL getResource(String type, String path) {
        //changed from ClassLoader.getSystemClassLoader() to this.getClass().getClassLoader()
        //because the first version does not work with Java Web Start
        //return ClassLoader.getSystemClassLoader().getResource(getFullPath(type, path));
        return this.getClass().getClassLoader().getResource(getFullPath(type, path));
    }

    /**
     * Returns the full path of a resource..
     *
     * @param type the logical type name
     * @param path der relative path + file name without extension
     * @return the resource's full path as String.
     */
    public String getFullPath(String type, String path) {
        ResourceModul rm = (ResourceModul)moduls.get(type);
        String p = resourceRootPath + "/" + rm.getPath();

        if (rm != null) {
            // construct path
            StringTokenizer st = new StringTokenizer(path, ".");
            while (st.hasMoreTokens()) {
                p = p + "/" + st.nextToken();
            }
            // append extension
            p = p + "." + rm.getExtension();

            return p;
        }

        return "";
    }

    /**
     * Type name for resource module: GIF
     */
    public static final String RESOURCE_GIF = "gif";

    /**
     * Predefined module for .gif-resources.
     */
    private static final class ResourceModulGIF implements ResourceModul {

        /**
         * Returns the resource's file extension: "gif".
         *
         * @return the file extension
         */
        public String getExtension() {
            return "gif";
        }

        /**
         * Returns the path relative to the current resource path.
         *
         * @return the path.
         */
        public String getPath() {
            return "gif";
        }

        /**
         * Returns the type name of the module. This is the name by which the resources are requested.
         *
         * @return the type name
         */
        public String getTypeName() {
            return RESOURCE_GIF;
        }
    }
}
